﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace HeuristicLab.Algorithms.Bandits.BanditPolicies {
  public class MeanAndVariancePolicyActionInfo : IBanditPolicyActionInfo {
    private bool disabled;
    public bool Disabled { get { return disabled; } }
    private OnlineMeanAndVarianceEstimator estimator = new OnlineMeanAndVarianceEstimator();
    private double knownValue;
    public int Tries { get { return estimator.N; } }
    public double SumReward { get { return estimator.Sum; } }
    public double AvgReward { get { return estimator.Avg; } }
    public double RewardVariance { get { return estimator.Variance; } }
    public double Value {
      get {
        if (disabled) return knownValue;
        else
          return AvgReward;
      }
    }

    public void UpdateReward(double reward) {
      Debug.Assert(!Disabled);
      estimator.UpdateReward(reward);
    }

    public void Disable(double reward) {
      disabled = true;
      this.knownValue = reward;
    }

    public void Reset() {
      disabled = false;
      knownValue = 0.0;
      estimator.Reset();
    }

    public void PrintStats() {
      Console.WriteLine("avg reward {0,5:F2} disabled {1}", AvgReward, Disabled);
    }
  }
}
